

def normalize_attribute_key(payload):
    key = [k for k in payload.keys() if k.endswith("Attributes")][0]
    payload['attributes'] = payload.pop(key)


class DecisionTask(object):

    def __init__(self, data):
        self.token = data['taskToken']
        self.events = data['events']

        self.workflow_id = data['workflowExecution']['workflowId']
        self.run_id = data['workflowExecution']['runId']
        self.workflow_type = data['workflowType']['name']
        self.workflow_version = data['workflowType']['version']

        self.decisions = []

    def __repr__(self):
        return '<DecisionTask %s v=%s id=%s>' % (self.workflow_type,
                                                 self.workflow_version,
                                                 self.workflow_id)

    @property
    def workflow_input(self):
        execution_event = self.last_event_type("WorkflowExecutionStarted")
        return execution_event['attributes'].get('input')

    def has_event_type(self, event_type):
        return bool(self.last_event_type(event_type))

    def last_event_type(self, event_type):
        events = [e for e in self.events if e['eventType'] == event_type]
        if events:
            event = events[-1]
            normalize_attribute_key(event)
            return event
        else:
            return None

    def add_decision(self, decision_type, **attributes):
        attribute_name = '%s%sDecisionAttributes' % (decision_type[0].lower(),
                                                     decision_type[1:])
        decision_payload = {
            'decisionType': decision_type,
            attribute_name: attributes,
        }
        self.decisions.append(decision_payload)



import logging

from caravan import Workflow

log = logging.getLogger(__name__)


class Test1(Workflow):

    name = 'Test1'
    version = '1.3'
    defaults = {
        'ExecutionStartToCloseTimeout': '3600',
        'TaskStartToCloseTimeout': '10',
        'TaskList': 'default',
        'ChildPolicy': 'TERMINATE',
        }

    def decide(self, task):
        log.info("Inside %s workflow", self.name)



import sys

from botocore.exceptions import ClientError

from caravan.swf import get_swf_connection, is_response_success
from caravan.commands import BaseCommand


class Command(BaseCommand):

    description = 'Start a workflow execution'

    def setup_arguments(self, parser):
        parser.add_argument('-d', '--domain', required=True)
        parser.add_argument('-i', '--id', required=True)
        parser.add_argument('-n', '--name', required=True)
        parser.add_argument('-v', '--version', required=True)
        parser.add_argument('-t', '--task-list')

        parser.add_argument(
            '--input',
            help='Raw input data for this execution',
            metavar='"<Some data>"')
        parser.add_argument(
            '--tag-list',
            help='List of tags to associate with the workflow execution',
            metavar='"tag1 tag2 ..."')
        parser.add_argument(
            '--execution-timeout',
            help='The maximum total duration for this workflow execution (seconds)')
        parser.add_argument(
            '--task-timeout',
            help='The maximum duration of decision tasks (seconds)')
        parser.add_argument(
            '--child-policy',
            help='Policy to use for the child executions of this execution')
        parser.add_argument(
            '--lambda-role',
            help='ARN of an IAM role that authorizes SWF to invoke Lambda functions')

    def run(self):
        connection = get_swf_connection()

        args = {}
        if self.args.input:
            args['input'] = self.args.input
        if self.args.task_list:
            args['task_list'] = self.args.task_list
        if self.args.tag_list:
            args['tagList'] = self.args.tag_list.split()
        if self.args.execution_timeout:
            args['executionStartToCloseTimeout'] = self.args.execution_timeout
        if self.args.task_timeout:
            args['taskStartToCloseTimeout'] = self.args.task_timeout
        if self.args.child_policy:
            args['childPolicy'] = self.args.child_policy
        if self.args.lambda_role:
            args['lambdaRole'] = self.args.lambda_role

        try:
            response = connection.start_workflow_execution(
                domain=self.args.domain,
                workflowId=self.args.id,
                workflowType={
                    'name': self.args.name,
                    'version': self.args.version
                    },
                **args)
        except ClientError as err:
            sys.exit(err)
        else:
            if is_response_success(response):
                print 'Workflow execution successfully created as RunId %s' % (
                    response['runId'])
            else:
                sys.exit('Error: %s' % response)



import boto3
from botocore.client import Config


def get_swf_connection():
    # Must increase the http timeout since SWF has a timeout of 60 sec
    config = Config(connect_timeout=50, read_timeout=70)
    connection = boto3.client("swf", region_name='us-east-1', config=config)
    return connection


def is_response_success(response):
    status_code = response.get('ResponseMetadata', {}).get('HTTPStatusCode')
    return status_code == 200



import argparse
import ConfigParser
import importlib
import inspect
import logging
import sys


class BaseCommand(object):

    """Base class for command-line tools."""

    default_config_section = 'caravan'

    @classmethod
    def main(cls):
        cmd = cls()
        cmd._parse_args()
        cmd._setup_logging()
        cmd._run()

    def _parse_args(self):
        # Config only parser
        config_parser = argparse.ArgumentParser(description=self.description,
                                                add_help=False)
        config_parser.add_argument('-c', '--config',
                                   help='config file for setup.')
        config_parser.add_argument('--config-section',
                                   default=self.default_config_section,
                                   help='section of the config file for setup.')
        args, remaining_args = config_parser.parse_known_args()

        # Full parser
        parser = argparse.ArgumentParser(parents=[config_parser])
        self._setup_base_arguments(parser)
        self.setup_arguments(parser)

        # Read defaults from config file
        if args.config:
            cp = ConfigParser.RawConfigParser()
            with open(args.config) as fp:
                cp.readfp(fp)
            config_items = cp.items(args.config_section)

            valid_options = [option_string
                             for action in parser._actions
                             for option_string in action.option_strings]

            for option, value in config_items:
                option_string = '--%s' % option
                if option_string in valid_options:
                    remaining_args.extend([option_string, value])

        self.args = parser.parse_args(remaining_args)

    def _setup_base_arguments(self, parser):
        parser.add_argument('--logging-config',
                            dest='logging_config',
                            help='Optional config file for logging.'
                                 ' Default to config_uri')
        parser.add_argument('--verbose',
                            dest='logging_level',
                            default=logging.WARNING,
                            action='store_const',
                            const=logging.INFO)
        parser.add_argument('--debug',
                            dest='logging_level',
                            default=logging.WARNING,
                            action='store_const',
                            const=logging.DEBUG)

    def _setup_logging(self):
        if self.args.logging_config:
            logging.config.fileConfig(self.args.logging_config)
        elif self.args.config:
            logging.config.fileConfig(self.args.config)
        else:
            logging.basicConfig(level=self.args.logging_level)

    def _run(self):
        logging.debug('Run command with args: %s', self.args)
        try:
            self.run()
        except KeyboardInterrupt:
            sys.exit(1)


def find_classes(module, cls):
    def predicate(obj):
        return inspect.isclass(obj) and issubclass(obj, cls) and obj is not cls
    members = inspect.getmembers(module, predicate)
    return [obj for name, obj in members]


class ClassesLoaderFromModule(object):

    """Load and return classes in a module that inherit from a class.

    This module must be in the python sys path.
    """

    def __init__(self, cls):
        self.cls = cls

    def __repr__(self):
        return '<ClassesLoader(%s)>' % self.cls.__name__

    def __call__(self, arg):
        sys.path.append('.')
        try:
            module = importlib.import_module(arg)
        finally:
            sys.path.pop()

        classes = find_classes(module, self.cls)
        if not classes:
            raise ValueError("No workflow in module %s" % arg)

        return classes



import sys

from botocore.exceptions import ClientError

from caravan.swf import get_swf_connection, is_response_success
from caravan.commands import BaseCommand


class Command(BaseCommand):

    description = 'Signal a workflow execution'

    def setup_arguments(self, parser):

        parser.add_argument('-d', '--domain', required=True)
        parser.add_argument('-i', '--id', required=True)
        parser.add_argument('--run-id')
        parser.add_argument('-s', '--signal', required=True)
        parser.add_argument(
            '--input',
            help='Raw input data for this signal',
            metavar='"<Some data>"')

    def run(self):
        connection = get_swf_connection()

        callargs = {}
        if self.args.input:
            callargs['input'] = self.args.input
        if self.args.run_id:
            callargs['runId'] = self.args.run_id

        try:
            response = connection.signal_workflow_execution(
                domain=self.args.domain,
                workflowId=self.args.id,
                signalName=self.args.signal,
                **callargs
                )
        except ClientError as err:
            sys.exit(err)
        else:
            if is_response_success(response):
                print 'Signal successfully sent'
            else:
                sys.exit('Error: %s' % response)



from setuptools import find_packages
from setuptools import setup


setup(
    name='caravan',
    version='0.0.2',
    description='Light python framework for AWS SWF',
    long_description=open('README.rst').read(),
    keywords='AWS SWF workflow distributed background task',
    author='Pior Bastida',
    author_email='pior@pbastida.net',
    url='https://github.com/pior/caravan',
    license='MIT',
    packages=find_packages(),
    zip_safe=False,
    install_requires=['boto3'],
    extras_require={
        'dev': ['zest.releaser[recommended]'],
        },
    entry_points={
        "console_scripts": [
            "caravan-decider = caravan.commands.decider:Command.main",
            "caravan-start = caravan.commands.start:Command.main",
            "caravan-signal = caravan.commands.signal:Command.main",
            ]
        },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Internet",
        "Programming Language :: Python :: 2.7",
        # "Programming Language :: Python :: 3.4",
        # "Programming Language :: Python :: 3.5"
        ],
    )



import logging
import pprint

from caravan.workers import get_default_identity
from caravan.models import DecisionTask

log = logging.getLogger(__name__)


class Worker(object):

    def __init__(self, connection, domain, task_list, workflows):
        self.conn = connection
        self.domain = domain
        self.task_list = task_list
        self.workflows = {w.name: w for w in workflows}
        self.identity = get_default_identity()

    def poll_swf(self):
        """Poll SWF for a decision task. Only return once actually get one."""
        task_list = dict(name=self.task_list)

        while True:
            resp = self.conn.poll_for_decision_task(domain=self.domain,
                                                    taskList=task_list,
                                                    identity=self.identity)

            task_token = resp.get('taskToken')
            if task_token:
                next_page_token = resp.get('nextPageToken')
                while next_page_token:
                    page = self.conn.poll_for_decision_task(
                        domain=self.domain,
                        taskList=task_list,
                        identity=self.identity,
                        nextPageToken=next_page_token)
                    next_page_token = page.get('nextPageToken')
                    resp['events'].extend(page['events'])

                return resp

    def run(self):
        log.info('Polling for decision task...')
        data = self.poll_swf()
        try:
            task = DecisionTask(data)
        except Exception as exc:
            log.error('Failed to prepare decision task: %s\nData:\n%s',
                      exc, pprint.pformat(data))
            raise

        log.info('Build decisions for task: %s', task)
        self.decide(task)

        log.info('Respond with decisions: %s', task.decisions)
        self.conn.respond_decision_task_completed(
            taskToken=task.token, decisions=task.decisions)

    def decide(self, task):
        workflow = self.workflows.get(task.workflow_type)
        if workflow:
            log.info('Run Workflow %s...', workflow)
            workflow().decide()
        else:
            log.warning('Unknown workflow %s', task.workflow_type)
            task.add_decision('FailWorkflowExecution',
                              reason='Unkown workflow type',
                              details='Workflow %s' % task.workflow_type)
